@@ -1,6 +1,6 @@
 package HTTP::Cookies;
 
-# Based on draft-ietf-http-state-man-mec-08.txt and
+# Based on draft-ietf-http-state-man-mec-03.txt and
 # http://www.netscape.com/newsref/std/cookie_spec.html
 
 use strict;
@@ -9,13 +9,7 @@ use HTTP::Headers::Util qw(split_header_words join_header_words);
 use LWP::Debug ();
 
 use vars qw($VERSION);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.18 $ =~ /(\d+)\.(\d+)/);
-
-my $EPOCH_OFFSET = 0;  # difference from Unix epoch
-if ($^O eq "MacOS") {
-    require Time::Local;
-    $EPOCH_OFFSET = Time::Local::timelocal(0,0,0,1,0,70);
-}
+$VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
 
 =head1 NAME
 
@@ -33,16 +27,16 @@ HTTP::Cookies - Cookie storage and management
 
 Cookies are a general mechanism which server side connections can use
 to both store and retrieve information on the client side of the
-connection.  For more information about cookies refer to
+connection.  For more information about cookies referrer to
 <URL:http://www.netscape.com/newsref/std/cookie_spec.html> and
 <URL:http://www.cookiecentral.com/>.  This module also implements the
-new style cookies described in I<RFC 2965>.
-The two variants of cookies are supposed to be able to coexist happily.
+new style cookies as described in I<draft-ietf-http-state-man-mec-03.txt>.
+The two variants of cookies is supposed to be able to coexist happily.
 
 Instances of the class I<HTTP::Cookies> are able to store a collection
-of Set-Cookie2: and Set-Cookie: headers and are able to use this
+of Set-Cookie2: and Set-Cookie:-headers and is able to use this
 information to initialize Cookie-headers in I<HTTP::Request> objects.
-The state of a I<HTTP::Cookies> object can be saved in and restored from
+The state of the I<HTTP::Cookies> can be saved and restored from
 files.
 
 =head1 METHODS
@@ -59,13 +53,12 @@ The following methods are provided:
 
 =item $cookie_jar = HTTP::Cookies->new;
 
-The constructor takes hash style parameters.  The following
+The constructor.  Takes hash style parameters.  The following
 parameters are recognized:
 
-  file:            name of the file to restore cookies from and save cookies to
-  autosave:        save during destruction (bool)
+  file:            name of the file to restore and save cookies to
+  autosave:        should we save during destruction (bool)
   ignore_discard:  save even cookies that are requested to be discarded (bool)
-  hide_cookie2:    don't add Cookie2 header to requests
 
 Future parameters might include (not yet implemented):
 
@@ -96,7 +89,7 @@ sub new
 
 The add_cookie_header() method will set the appropriate Cookie:-header
 for the I<HTTP::Request> object given as argument.  The $request must
-have a valid url attribute before this method is called.
+have a valid url() attribute before this method is called.
 
 =cut
 
@@ -106,18 +99,16 @@ sub add_cookie_header
     my $request = shift || return;
     my $url = $request->url;
     my $domain = $url->host;
-    $domain = "$domain.local" unless $domain =~ /\./;
     my $secure_request = ($url->scheme eq "https");
-    my $req_path = _url_path($url);
+    my $req_path = $url->epath;
     my $req_port = $url->port;
     my $now = time();
-    _normalize_path($req_path) if $req_path =~ /%/;
+    $self->_normalize_path($req_path) if $req_path =~ /%/;
 
     my @cval;    # cookie values for the "Cookie" header
     my $set_ver;
-    my $netscape_only = 0; # An exact domain match applies to any cookie
 
-    while ($domain =~ /\./) {
+    while (($domain =~ tr/././) >= 2) {   # must be at least 2 dots
 
         LWP::Debug::debug("Checking $domain for cookies");
 	my $cookies = $self->{COOKIES}{$domain};
@@ -146,28 +137,17 @@ sub add_cookie_header
 		    next;
 		}
 		if ($port) {
+		    #XXX: must also handle empty port ""
 		    my $found;
-		    if ($port =~ s/^_//) {
-			# The correponding Set-Cookie attribute was empty
-			$found++ if $port eq $req_port;
-			$port = "";
-		    } else {
-			my $p;
-			for $p (split(/,/, $port)) {
-			    $found++, last if $p eq $req_port;
-			}
+		    my $p;
+		    for $p (split(/,/, $port)) {
+			$found++, last if $p eq $req_port;
 		    }
 		    unless ($found) {
 		        LWP::Debug::debug("   port $port:$req_port does not fit");
 			next;
 		    }
 		}
-		if ($version > 0 && $netscape_only) {
-		    LWP::Debug::debug("   domain $domain applies to " .
-				      "Netscape-style cookies only");
-		    next;
-		}
-
 	        LWP::Debug::debug("   it's a match");
 
 		# set version number of cookie header.
@@ -176,8 +156,8 @@ sub add_cookie_header
 		if (!$set_ver++) {
 		    if ($version >= 1) {
 			push(@cval, "\$Version=$version");
-		    } elsif (!$self->{hide_cookie2}) {
-			$request->header(Cookie2 => '$Version="1"');
+		    } else {
+			$request->header(Cookie2 => "\$Version=1");
 		    }
 		}
 
@@ -203,22 +183,8 @@ sub add_cookie_header
         }
 
     } continue {
-	# Try with a more general domain, alternately stripping
-	# leading name components and leading dots.  When this
-	# results in a domain with no leading dot, it is for
-	# Netscape cookie compatibility only:
-	#
-	# a.b.c.net	Any cookie
-	# .b.c.net	Any cookie
-	# b.c.net	Netscape cookie only
-	# .c.net	Any cookie
-
-	if ($domain =~ s/^\.+//) {
-	    $netscape_only = 1;
-	} else {
-	    $domain =~ s/[^.]*//;
-	    $netscape_only = 0;
-	}
+	# Try with a more general domain:  www.sol.no ==> .sol.no
+	$domain =~ s/^\.?[^.]*//;
     }
 
     $request->header(Cookie => join("; ", @cval)) if @cval;
@@ -230,8 +196,8 @@ sub add_cookie_header
 =item $cookie_jar->extract_cookies($response);
 
 The extract_cookies() method will look for Set-Cookie: and
-Set-Cookie2: headers in the I<HTTP::Response> object passed as
-argument.  Any of these headers that are found are used to update
+Set-Cookie2:-headers in the I<HTTP::Response> object passed as
+argument.  If some of these headers are found they are used to update
 the state of the $cookie_jar.
 
 =cut
@@ -240,36 +206,30 @@ sub extract_cookies
 {
     my $self = shift;
     my $response = shift || return;
-
     my @set = split_header_words($response->_header("Set-Cookie2"));
-    my @ns_set = $response->_header("Set-Cookie");
-
-    return $response unless @set || @ns_set;  # quick exit
+    my $netscape_cookies;
+    unless (@set) {
+	@set = $response->_header("Set-Cookie");
+	return $response unless @set;
+	$netscape_cookies++;
+    }
 
     my $url = $response->request->url;
     my $req_host = $url->host;
-    $req_host = "$req_host.local" unless $req_host =~ /\./;
     my $req_port = $url->port;
-    my $req_path = _url_path($url);
-    _normalize_path($req_path) if $req_path =~ /%/;
-
-    if (@ns_set) {
+    my $req_path = $url->epath;
+    $self->_normalize_path($req_path) if $req_path =~ /%/;
+    
+    if ($netscape_cookies) {
 	# The old Netscape cookie format for Set-Cookie
         # http://www.netscape.com/newsref/std/cookie_spec.html
 	# can for instance contain an unquoted "," in the expires
 	# field, so we have to use this ad-hoc parser.
 	my $now = time();
-
-	# Build a hash of cookies that was present in Set-Cookie2
-	# headers.  We need to skip them if we also find them in a
-	# Set-Cookie header.
-	my %in_set2;
-	for (@set) {
-	    $in_set2{$_->[0]}++;
-	}
-
+	my @old = @set;
+	@set = ();
 	my $set;
-	for $set (@ns_set) {
+	for $set (@old) {
 	    my @cur;
 	    my $param;
 	    my $expires;
@@ -278,21 +238,15 @@ sub extract_cookies
 		#print "$k => $v\n";
 		my $lc = lc($k);
 		if ($lc eq "expires") {
-		    my $etime = str2time($v);
-		    if ($etime) {
-			push(@cur, "Max-Age" => str2time($v) - $now);
-			$expires++;
-		    }
+		    push(@cur, "Max-Age" => str2time($v) - $now);
+		    $expires++;
 		} else {
 		    push(@cur, $k => $v);
 		}
 	    }
-	    next if $in_set2{$cur[0]};
-
-#	    push(@cur, "Port" => $req_port);
+	    push(@cur, "Port" => $req_port);
 	    push(@cur, "Discard" => undef) unless $expires;
 	    push(@cur, "Version" => 0);
-	    push(@cur, "ns-cookie" => 1);
 	    push(@set, \@cur);
 	}
     }
@@ -310,29 +264,25 @@ sub extract_cookies
 	while (@$set) {
 	    my $k = shift @$set;
 	    my $v = shift @$set;
+	    $v = 1 unless defined $v;
 	    my $lc = lc($k);
 	    # don't loose case distinction for unknown fields
 	    $k = $lc if $lc =~ /^(?:discard|domain|max-age|
                                     path|port|secure|version)$/x;
-	    if ($k eq "discard" || $k eq "secure") {
-		$v = 1 unless defined $v;
-	    }
 	    next if exists $hash{$k};  # only first value is signigicant
 	    $hash{$k} = $v;
 	};
 
 	my %orig_hash = %hash;
 	my $version   = delete $hash{version};
-	$version = 1 unless defined($version);
 	my $discard   = delete $hash{discard};
 	my $secure    = delete $hash{secure};
 	my $maxage    = delete $hash{'max-age'};
-	my $ns_cookie = delete $hash{'ns-cookie'};
 
 	# Check domain
 	my $domain  = delete $hash{domain};
 	if (defined($domain) && $domain ne $req_host) {
-	    if ($domain !~ /\./ && $domain ne "local") {
+	    unless ($domain =~ /\./) {
 	        LWP::Debug::debug("Domain $domain contains no dot");
 		next SET_COOKIE;
 	    }
@@ -347,7 +297,7 @@ sub extract_cookies
 		next SET_COOKIE;
 	    }
 	    my $hostpre = substr($req_host, 0, length($req_host) - $len);
-	    if ($hostpre =~ /\./ && !$ns_cookie) {
+	    if ($hostpre =~ /\./ && !$netscape_cookies) {
 	        LWP::Debug::debug("Host prefix contain a dot: $hostpre => $domain");
 		next SET_COOKIE;
 	    }
@@ -357,10 +307,10 @@ sub extract_cookies
 
 	my $path = delete $hash{path};
 	my $path_spec;
-	if (defined $path && $path ne '') {
+	if (defined $path) {
 	    $path_spec++;
-	    _normalize_path($path) if $path =~ /%/;
-	    if (!$ns_cookie &&
+	    $self->_normalize_path($path) if $path =~ /%/;
+	    if (!$netscape_cookies &&
                 substr($req_path, 0, length($path)) ne $path) {
 	        LWP::Debug::debug("Path $path is not a prefix of $req_path");
 		next SET_COOKIE;
@@ -374,8 +324,9 @@ sub extract_cookies
 	my $port;
 	if (exists $hash{port}) {
 	    $port = delete $hash{port};
-	    if (defined $port) {
-		$port =~ s/\s+//g;
+	    $port = "" unless defined $port;
+	    $port =~ s/\s+//g;
+	    if (length $port) {
 		my $found;
 		for my $p (split(/,/, $port)) {
 		    unless ($p =~ /^\d+$/) {
@@ -388,8 +339,6 @@ sub extract_cookies
 		    LWP::Debug::debug("Request port ($req_port) not found in $port");
 		    next SET_COOKIE;
 		}
-	    } else {
-		$port = "_$req_port";
 	    }
 	}
 	$self->set_cookie($version,$key,$val,$path,$domain,$port,$path_spec,$secure,$maxage,$discard, \%hash)
@@ -407,8 +356,8 @@ The set_cookie() method updates the state of the $cookie_jar.  The
 $key, $val, $domain, $port and $path arguments are strings.  The
 $path_spec, $secure, $discard arguments are boolean values. The $maxage
 value is a number indicating number of seconds that this cookie will
-live.  A value <= 0 will delete this cookie.  %rest defines
-various other attributes like "Comment" and "CommentURL".
+live.  A value <= 0 will delete this cookie.  The %rest are a place
+for various other attributes like "Comment" and "CommentURL".
 
 =cut
 
@@ -419,13 +368,16 @@ sub set_cookie
        $key, $val, $path, $domain, $port,
        $path_spec, $secure, $maxage, $discard, $rest) = @_;
 
+    # there must always be at least 2 dots in a domain
+    return $self if ($domain =~ tr/././) < 2;
+
     # path and key can not be empty (key can't start with '$')
     return $self if !defined($path) || $path !~ m,^/, ||
 	            !defined($key)  || $key  !~ m,[^\$],;
 
     # ensure legal port
     if (defined $port) {
-	return $self unless $port =~ /^_?\d+(?:,\d+)*$/;
+	return $self unless $port eq "" || $port =~ /^\d+(?:,\d+)*$/;
     }
 
     my $expires;
@@ -451,15 +403,15 @@ sub set_cookie
 
 =item $cookie_jar->save( [$file] );
 
-This method file saves the state of the $cookie_jar to a file.
+Calling this method file save the state of the $cookie_jar to a file.
 The state can then be restored later using the load() method.  If a
 filename is not specified we will use the name specified during
 construction.  If the attribute I<ignore_discared> is set, then we
 will even save cookies that are marked to be discarded.
 
-The default is to save a sequence of "Set-Cookie3" lines.
+The default is to save a sequence of "Set-Cookie3" lines.  The
 "Set-Cookie3" is a proprietary LWP format, not known to be compatible
-with any browser.  The I<HTTP::Cookies::Netscape> sub-class can
+with any other browser.  The I<HTTP::Cookies::Netscape> sub-class can
 be used to save in a format compatible with Netscape.
 
 =cut
@@ -478,7 +430,7 @@ sub save
 
 =item $cookie_jar->load( [$file] );
 
-This method reads the cookies from the file and adds them to the
+This method will read the cookies from the file and add them to the
 $cookie_jar.  The file must be in the format written by the save()
 method.
 
@@ -489,7 +441,6 @@ sub load
     my $self = shift;
     my $file = shift || $self->{'file'} || return;
     local(*FILE, $_);
-    local $/ = "\n";  # make sure we got standard record separator
     open(FILE, $file) or return;
     my $magic = <FILE>;
     unless ($magic =~ /^\#LWP-Cookies-(\d+\.\d+)/) {
@@ -530,8 +481,7 @@ sub load
 
 =item $cookie_jar->revert;
 
-This method empties the $cookie_jar and re-loads the $cookie_jar
-from the last save file.
+Will revert to the state of last save.
 
 =cut
 
@@ -547,7 +497,7 @@ sub revert
 Invoking this method without arguments will empty the whole
 $cookie_jar.  If given a single argument only cookies belonging to
 that domain will be removed.  If given two arguments, cookies
-belonging to the specified path within that domain are removed.  If
+belonging to the specified path within that domain is removed.  If
 given three arguments, then the cookie with the specified key, path
 and domain is removed.
 
@@ -581,7 +531,7 @@ sub DESTROY
 =item $cookie_jar->scan( \&callback );
 
 The argument is a subroutine that will be invoked for each cookie
-stored in the $cookie_jar.  The subroutine will be invoked with
+stored within the $cookie_jar.  The subroutine will be invoked with
 the following arguments:
 
   0  version
@@ -620,7 +570,7 @@ sub scan
 
 The as_string() method will return the state of the $cookie_jar
 represented as a sequence of "Set-Cookie3" header lines separated by
-"\n".  If $skip_discard is TRUE, it will not return lines for
+"\n".  If given a argument that is TRUE, it will not return lines for
 cookies with the I<Discard> attribute.
 
 =cut
@@ -652,22 +602,14 @@ sub as_string
 }
 
 
-sub _url_path
-{
-    my $url = shift;
-    my $path = eval { $url->epath };    # URI::URL method
-    $path = $url->path if $@;           # URI::_generic method
-    $path = "/" unless length $path;
-    $path;
-}
-
 sub _normalize_path  # so that plain string compare can be used
 {
+    shift;  # $self
     my $x;
     $_[0] =~ s/%([0-9a-fA-F][0-9a-fA-F])/
 	         $x = uc($1);
                  $x eq "2F" || $x eq "25" ? "%$x" :
-                                            pack("C", hex($x));
+                                            pack("c", hex($x));
               /eg;
     $_[0] =~ s/([\0-\x20\x7f-\xff])/sprintf("%%%02X",ord($1))/eg;
 }
@@ -678,8 +620,8 @@ sub _normalize_path  # so that plain string compare can be used
 
 =head1 SUB CLASSES
 
-We also provide a subclass called I<HTTP::Cookies::Netscape> which
-loads and saves Netscape compatible cookie files.  You
+We also provide a subclass called I<HTTP::Cookies::Netscape> which make
+cookie loading and saving compatible with Netscape cookie files.  You
 should be able to have LWP share Netscape's cookies by constructing
 your $cookie_jar like this:
 
@@ -690,7 +632,7 @@ your $cookie_jar like this:
 
 Please note that the Netscape cookie file format is not able to store
 all the information available in the Set-Cookie2 headers, so you will
-probably loose some information if you save in this format.
+probably loose some information if you save using this format.
 
 =cut
 
@@ -704,7 +646,6 @@ sub load
     my($self, $file) = @_;
     $file ||= $self->{'file'} || return;
     local(*FILE, $_);
-    local $/ = "\n";  # make sure we got standard record separator
     my @cookies;
     open(FILE, $file) || return;
     my $magic = <FILE>;
@@ -713,7 +654,7 @@ sub load
 	close(FILE);
 	return;
     }
-    my $now = time() - $EPOCH_OFFSET;
+    my $now = time();
     while (<FILE>) {
 	next if /^\s*\#/;
 	next if /^\s*$/;
@@ -741,12 +682,12 @@ sub save
 
 EOT
 
-    my $now = time - $EPOCH_OFFSET;
+    my $now = time;
     $self->scan(sub {
 	my($version,$key,$val,$path,$domain,$port,
 	   $path_spec,$secure,$expires,$discard,$rest) = @_;
 	return if $discard && !$self->{ignore_discard};
-	$expires = $expires ? $expires - $EPOCH_OFFSET : 0;
+	$expires ||= 0;
 	return if $now > $expires;
 	$secure = $secure ? "TRUE" : "FALSE";
 	my $bool = $domain =~ /^\./ ? "TRUE" : "FALSE";
@@ -762,7 +703,7 @@ __END__
 
 =head1 COPYRIGHT
 
-Copyright 1997-1999 Gisle Aas
+Copyright 1997, Gisle Aas
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
